from dataclasses import dataclass, field
from datetime import datetime


@dataclass
class ResultBundle:
    """Class that holds a response result and its properties."""

    url: str
    source: str
    results: list[dict] = field(default_factory=list, repr=False)
    created_at: datetime = field(default_factory=datetime.now)



import aiofiles

import json

from peneira.sources.open_alex import map_to_bibtex_type


def to_json(capsule):
    return json.dumps(capsule)


def to_bibtex(capsule):
    bibtex_dict = map_to_bibtex_type(capsule)

    bibtex_str = f"@{bibtex_dict['type']}{{{bibtex_dict['id']},\n"
    for key, value in bibtex_dict.items():
        if key != "id" and key != "type":
            bibtex_str += f"  {key} = {{{value}}},\n"
    bibtex_str += "}\n\n"

    return bibtex_str


async def write_results_to_file(result_bundle, filename, output_format_func=to_json):
    async with aiofiles.open(filename, "a") as file:
        for result in result_bundle.results:
            capsule = {
                "source": result_bundle.source,
                "url": result_bundle.url,
                "result": result,
                "created_at": str(result_bundle.created_at),
            }

            await file.write(f"{output_format_func(capsule)}\n")
    return result_bundle.results



import math

import httpx

from peneira import setup_logger
from peneira.sources import ResultBundle
from aiolimiter import AsyncLimiter


logger = setup_logger(__name__)
BASE_URL = "https://api.openalex.org"
WORKS_PER_PAGE = 200
SOURCE = "open_alex"

# allow for 10 requests per second
# https://docs.openalex.org/how-to-use-the-api/rate-limits-and-authentication
rate_limiter = AsyncLimiter(10)


async def fetch_papers(query, page=1):
    params = {"search": query, "page": page, "per-page": WORKS_PER_PAGE}
    async with rate_limiter:  # wait for a slot to be available
        async with httpx.AsyncClient() as client:
            response = await client.get(f"{BASE_URL}/works", params=params)
            results = response.json().get("results", [])
            return ResultBundle(url=str(response.url), source=SOURCE, results=results)


async def establish_number_of_pages(query):
    params = {"search": query}
    async with httpx.AsyncClient() as client:
        response = await client.get(f"{BASE_URL}/works", params=params)
        payload = response.json()
        total = payload["meta"]["count"]
        pages = total / WORKS_PER_PAGE
        if pages != 1 and pages % 2 != 0:
            pages += 1
        return math.trunc(pages), total


def map_to_bibtex_type(capsule):
    result = capsule["result"]
    authors_list = []
    institutions_list = []
    countries = []
    authorships = result.get("authorships", [{}])
    for authorship in authorships:
        authors_list.append(authorship.get("author", {}).get("display_name", ""))
        if authorship.get("institutions"):
            institutions_list.extend(
                [
                    institution.get("display_name", "")
                    for institution in authorship.get("institutions")
                ]
            )
        if authorship.get("countries"):
            countries.extend(authorship["countries"])

    authors = " and ".join(authors_list)
    institutions = ", ".join(institutions_list)
    countries = ", ".join(sorted(set(countries)))
    all_keywords = []
    all_keywords.extend(
        [keyword["display_name"] for keyword in result.get("keywords", [])]
    )
    all_keywords.extend(
        [keyword["descriptor_name"] for keyword in result.get("mesh", [])]
    )
    all_keywords.extend(
        [keyword["display_name"] for keyword in result.get("concepts", [])]
    )
    all_keywords = ", ".join(sorted(set(all_keywords)))

    return {
        "id": result.get("id", ""),
        "doi": result.get("doi", ""),
        "title": result.get("title", ""),
        "year": result.get("publication_year", ""),
        "date": result.get("publication_date", ""),
        "authors": authors,
        "institutions": institutions,
        "country": countries,
        "language": result.get("language", ""),
        "type": result.get("type", ""),
        "open_access": result.get("open_access", {}).get("oa_status", ""),
        "abstract": "",  # this source does not provide abstracts
        "keywords": all_keywords,
        "url": capsule.get("url", ""),
        "source": capsule.get("source", ""),
        "created_at": capsule.get("created_at", ""),
    }


async def search_open_alex(query):
    number_of_pages, total = await establish_number_of_pages(query)
    logger.info(
        f"Fetching articles for OPEN_ALEX... {total} papers "
        f"distributed in {number_of_pages} pages."
    )
    tasks = [fetch_papers(query, page) for page in range(1, number_of_pages + 1)]
    return tasks



import logging


def setup_logger(name):
    logger = logging.getLogger(name)
    logger.setLevel(logging.INFO)
    stream_handler = logging.StreamHandler()
    stream_handler.setLevel(logging.INFO)
    stream_handler.setFormatter(logging.Formatter("%(module)s: %(message)s"))
    logger.addHandler(stream_handler)
    return logger



import asyncio

import asyncclick as click

from peneira.exporters import write_results_to_file, to_json, to_bibtex
from peneira.sources.open_alex import search_open_alex
from peneira.sources.semantic_scholar import search_semantic_scholar

sources_search_func = {
    "open_alex": search_open_alex,
    "semantic_scholar": search_semantic_scholar,
}


@click.group()
async def cli():
    pass


@cli.command()
@click.option(
    "--filename",
    "-f",
    default="results.json",
    help="Filename with extension. Example: -f results.bib",
)
@click.option(
    "--sources",
    "-s",
    multiple=True,
    default=["open_alex"],
    help="Articles sources. Options: open_alex, semantic_scholar.",
    show_default=True,
)
@click.option(
    "--output", "-o", default="json", help="Output format. Options: json, bibtex."
)
async def cli(filename, sources, output):
    """Fetch articles from different sources using given QUERY."""
    if output.lower() == "bibtex":
        output_format_func = to_bibtex
    elif output.lower() == "json":
        output_format_func = to_json
    else:
        raise ValueError(f"Unsupported format {output}")

    all_tasks = []
    for source in sources:
        search_string = click.prompt(f"Please enter the search string for {source}")
        try:
            all_tasks.extend(await sources_search_func[source](search_string))
        except ValueError:
            raise ValueError(f"Unsupported source {source}")

    click.echo("Executing the search...")
    results = await asyncio.gather(*all_tasks)

    for result_bundle in results:
        await write_results_to_file(result_bundle, filename, output_format_func)
    click.echo("Done.")


def main():
    asyncio.run(cli())


if __name__ == "__main__":
    main()



from dataclasses import dataclass

import httpx
from aiolimiter import AsyncLimiter

from peneira import setup_logger
from peneira.sources import ResultBundle


logger = setup_logger(__name__)

BASE_URL = "https://api.semanticscholar.org/graph/v1/"
SOURCE = "semantic_scholar"
SEMANTIC_SCHOLAR_FIELDS = (
    "url,title,venue,year,authors,externalIds,abstract,openAccessPdf,"
    "fieldsOfStudy,publicationTypes,journal"
)

# with each unauthenticated IP limited to 1 request per second
# https://www.semanticscholar.org/product/api
rate_limiter = AsyncLimiter(60)  # 60 request per minute


@dataclass
class SemanticScholar:
    query: str
    _token: str = None

    def __post_init__(self):
        self._fields = SEMANTIC_SCHOLAR_FIELDS

    async def search(self, _result_bundle=None):
        params = {"fields": self._fields}
        if self._token:
            params["token"] = self._token

        async with rate_limiter:  # wait for a slot to be available
            async with httpx.AsyncClient() as client:
                response = await client.get(
                    f"{BASE_URL}paper/search/bulk?query={self.query}&", params=params
                )
                self._token = response.json().get("token")

                results = response.json().get("data", [])
                if _result_bundle is None:
                    _result_bundle = ResultBundle(
                        url=str(response.url), source=SOURCE, results=results
                    )
                else:
                    _result_bundle.results.extend(results)

                if self._token:
                    return await self.search(_result_bundle=_result_bundle)

                return _result_bundle


async def search_semantic_scholar(query):
    logger.info("Fetching articles for SEMANTIC_SCHOLAR...")
    semantic_scholar = SemanticScholar(query=query)
    return [semantic_scholar.search()]
